﻿#include "pch.h"
#include "resource.h"

HWND _hwnd = NULL;
cv::Mat _cv_mat;
RGBTRIPLE* _rgb_triple = nullptr;
unsigned int _bi_pocet_bitu = 0;

inline const wchar_t* trida_hlavni() noexcept
{
	return L"opencv_winapi";
}

inline void __declspec(noreturn) kriticke_ukonceni() noexcept
{
	_CrtDbgBreak();
	FatalAppExit(0, L"Došlo k závažné chybě! Aplikace bude ukončena...");
}

void otevrit_soubor(const char* sz_cesta) noexcept
{
	if (!_cv_mat.empty())
		_cv_mat.release();
	_cv_mat = cv::imread(sz_cesta, -1);
	if (_cv_mat.empty())
		return;
	_bi_pocet_bitu = (int)((_cv_mat.dataend - _cv_mat.datastart) / (_cv_mat.cols * _cv_mat.rows) * 8);
	RedrawWindow(_hwnd, nullptr, nullptr, RDW_ERASE | RDW_INVALIDATE | RDW_ERASENOW);
}

void vybrat_soubor() noexcept
{
	char* soubor = (char*)malloc(32768);
	soubor[0] = 0;
	OPENFILENAMEA ofn;
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = _hwnd;
	ofn.lpstrFile = soubor;
	ofn.nMaxFile = 32768;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
	if (!GetOpenFileNameA(&ofn))
	{
		free(soubor);
		return;
	}
	otevrit_soubor(soubor);
	free(soubor);
}

void obr_blur() noexcept
{
	if (_cv_mat.empty())
		return;
	cv::blur(_cv_mat, _cv_mat, cv::Size(15, 15));
	RedrawWindow(_hwnd, nullptr, nullptr, RDW_ERASE | RDW_INVALIDATE | RDW_ERASENOW);
}

void wm_paint(HWND hwnd) noexcept
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hwnd, &ps);
	RECT rect;
	GetClientRect(hwnd, &rect);
	BITMAPINFO bmi = { 0 };
	bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biWidth = _cv_mat.cols;
	bmi.bmiHeader.biHeight = _cv_mat.rows * -1;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = _bi_pocet_bitu;
	StretchDIBits(hdc, 0, 0, rect.right - rect.left, rect.bottom - rect.top,
		0, 0, bmi.bmiHeader.biWidth, abs(bmi.bmiHeader.biHeight),
		(RGBTRIPLE*)_cv_mat.data,
		&bmi, DIB_RGB_COLORS, SRCCOPY);
	EndPaint(hwnd, &ps);
}

LRESULT CALLBACK window_proc(HWND hwnd, UINT zprava, WPARAM wparam, LPARAM lparam) noexcept
{
	switch (zprava)
	{
	case WM_COMMAND:
		switch (LOWORD(wparam))
		{
		case ID_OBR_BLUR:
			obr_blur();
			break;
		case ID_SOUBOR_OTEVRIT:
			vybrat_soubor();
			break;
		case ID_KONEC:
			PostMessage(hwnd, WM_CLOSE, 0, 0);
			break;
		}
		break;
	case WM_PAINT:
		if (!_cv_mat.empty())
			wm_paint(hwnd);
		else
			return DefWindowProc(hwnd, zprava, wparam, lparam);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hwnd, zprava, wparam, lparam);
	}
	return 0;
}

int APIENTRY wWinMain(_In_ HINSTANCE hinstance, _In_opt_ HINSTANCE, _In_ LPWSTR, _In_ int)
{
	WNDCLASSEXW wcex;
	memset(&wcex, 0, sizeof(wcex));
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = window_proc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = hinstance;
	wcex.hIcon = LoadIcon(hinstance, MAKEINTRESOURCE(IDI_HLAVNI));
	wcex.hCursor = LoadCursor(nullptr, IDC_ARROW);
	wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wcex.lpszClassName = trida_hlavni();
	wcex.lpszMenuName = MAKEINTRESOURCEW(IDR_HLAVNI);
	wcex.hIconSm = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_HLAVNI));
	if (!RegisterClassExW(&wcex))
		kriticke_ukonceni();
	_hwnd = CreateWindowEx(0, trida_hlavni(), L"OpenCV ve WinAPI", WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, nullptr, nullptr, hinstance, nullptr);
	if (!_hwnd)
		kriticke_ukonceni();
	ShowWindow(_hwnd, SW_SHOW);
	UpdateWindow(_hwnd);
	vybrat_soubor();
	MSG msg;
	while (GetMessage(&msg, nullptr, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int)msg.wParam;
}

